%Release of Gaberson Software.  Disclaimer: this program is furnished by 
%Howie Gaberson and is accepted and used by the recipient with the express 
%understanding that Howie makes no warranty, expressed or implied, 
%concerning the accuracy, completeness, reliability, usability, or 
%suitability for any particular purpose of the information and data contained 
%in this program or furnished in connection therewith, and Howie shall be 
%under no liability whatsoever to any person by reason of any use made 
%thereof.  The program belongs to Howie. Therefore, the recipient 
%further agrees not to assert any proprietary rights therein or to represent 
%this program to anyone as other than a program Howie wrote.

%**** PVposneg.m ***** 090906 
%by Howard A. Gaberson, 234 Corsicana Dr.; Oxnard, CA 93036-1300
%(805)485-5307, hagaberson@att.net

%Calculates the positive and negative PVSS per Gaberson algorithm presnted in "SHOCK 
%SPECTRUM CALCULATION FROM ACCELERATION TIME HISTORIES" Technical Note TN no.
%N-1590 Civil Engineering Laboratory, Naval Construction Battalion Center, 
%Port Hueneme, CA 93043.  1980;  AD A097162, available from, www.dtic.mil;
%or  www.ntis.gov
%
%Program expects file YYDD of accelerations in g in workspace.  
%Values for flow, fhigh, fs, fpd, zeta are edited into here.
nv=length(YYDD);% nv= number of values in shock
YYDD=YYDD(:);%assure YYDD a column
%*********you must set a low frequency here. ***UNDERFLOW ERROR IF FS/FLOW >=720,000
flow=.1;
%*********you must set a high freq here
fhigh=5000;
%********* Now spec a SAMPLING RATE (SAMPLES/SEC.)
fs=50000;
%*********Now set FREQS PER DECADE, (ABT 100-200, BUT WHATEVER)
fpd=100;
%*********Insert your damping ratio here with a value for zeta
zeta= 0.05;
flowlog=log10(flow);
c2=round(flowlog)-1;
jlow=fpd*(flowlog-c2);
jstart=fix(jlow);
if(jlow ~= jstart);
	jstart=jstart+1;
end;
fstart=10.^(jstart/fpd+c2);
jstop=round(fpd*(log10(fhigh)-c2));
fstop=10^(jstop/fpd+c2);
nfreqs=jstop-jstart+1;
tpi=2*pi;
G=386.087;
yy=G*YYDD;
h=1/fs;
eta=sqrt(1-zeta^2);
tzeta=2.*zeta;
om2zs=1.-tzeta*zeta;
zmin=zeros(size(1:nfreqs)); zmax=zmin;
f=10.^((jstart:jstop)./fpd+c2);

for jj=1:nfreqs;
	wom=tpi*f(jj);
	wh=wom*h;
	eps=exp(-zeta*wh);
	a15=-eps*eps;
	ewh=eta*wh;
	kap=cos(ewh);
	sig=sin(ewh);
	k1=om2zs*eps*sig/eta;
	ek=eps*kap;
	a16=2*ek;
	w3h=1/(wh*wom*wom);
	a24=w3h*(tzeta*ek+a15*(tzeta+wh)+k1);
	a25=2*w3h*(wh*ek-k1-zeta*(1+a15));
	a26=w3h*(k1+tzeta*(1-ek)-wh);
	% zadd is number of zeros to append, to get residual spectrum. 
	zadd=ceil(fs/f(jj)); yytemp=[yy; zeros(zadd,1)];
	%XDD=filter(b,a,YDDtemp);
	%XDDMAX(n,1)=max(abs((XDD)));

	z=filter([a26 a25 a24],[1 -a16 -a15],yytemp);

	zmax=max(z); zmin=min(z);
	pvmin(jj)=wom*abs(zmin);
	pvmax(jj)=wom*abs(zmax);
end

loglog(f,pvmin,f,pvmax)
handle1=line(f,pvmax);
set(handle1,'linew',2,'color','k')
handle2=line(f,pvmin);
set(handle2,'linew',2,'color','r')
%
%To apply fourcp for four coordinate paper.
%type hold on;fourcp;hold off;
